# Epsilon Example

In this example we will first train a Support Vector Machine model on the epsilon dataset, using both `scikit-learn` and `snap-ml-local`. 
Then we will train a Boosting model using `xgboost` and `snap-ml-local`.

The epsilon dataset is from the [PASCAL Large Scale Learning Challenge](http://www.k4all.org/project/large-scale-learning-challenge/). 


## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/epsilon_normalized.bz2
bunzip2 epsilon_normalized.bz2
cd ../
```

## Data Pre-processing
The data is in SvmLight format which is not really appropriate since this dataset is dense. Therefore, we perform 
some pre-processing. 

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/preprocess-epsilon.py`

Command to pre-process the data available in specific directory:
`python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/preprocess-epsilon.py  --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

## Training and Evaluating a Support Vector Machine Model

One can run the entire example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon.py`

Command to run example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

Command to run example on GPU 0:
python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon.py --use_gpu

Command to run example on specified GPUs:
python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon.py --use_gpu --device_ids 1 2


Command to run example with fit_intercept:
python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon.py --use_gpu --fit_intercept

Command to collect training information:
```
python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon.py --use_gpu --return_training_history=full

python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon.py --use_gpu --return_training_history=summary
```

One should see the the following output on a P9+V100 system (--use_gpu --device_ids 0):

```
Data load time (s):  1.49
[pai4sk] GPU Training time (s):  3.76
[pai4sk] GPU Accuracy: 0.898
[pai4sk] GPU Hinge-loss: 0.273
[pai4sk.svm] GPU Training time (s):  3.48
[pai4sk.svm] GPU Accuracy: 0.898
[pai4sk.svm] GPU Hinge-loss: 0.273
[sklearn] Training time (s):  23.06
[sklearn] Accuracy:   0.898
[sklearn] Hinge loss:   0.273
```

Times will vary depending on system architecture.

Note: By default the training will run on CPU.
With --use_gpu option, default device_ids list contains only GPU 0 for pai4sk and all the available GPUs for pai4sk.lmodel.


## Training and Evaluating a Boosting Machine Model

One can run the entire example by executing `export OMP_PROC_BIND=spread ; numactl -N0 -m0 python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon-boosting.py`.

Command to run the example using the pre-processed data in a specific directory.
`export OMP_PROC_BIND=spread ; numactl -N0 -m0 python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon-boosting.py --data_path path/to/data`.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

Command to run the example using the pre-processed data in a specific directory using a specific number of CPU threads (e.g., 16 threads).
`export OMP_PROC_BIND=spread ; numactl -N0 -m0 python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon-boosting.py --data_path path/to/data --num_threads 16`

Command to run the example using the pre-processed data in a specific directory using a specific number of CPU threads (e.g., 16 threads) and one GPU.
`export OMP_PROC_BIND=spread ; numactl -N0 -m0 python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon-boosting.py --data_path path/to/data --num_threads 16 --use_gpu`

After running the following command on a P9 system ``export OMP_PROC_BIND=spread ; numactl -N0 -m0 python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon-boosting.py --data_path path/to/data --num_threads 16``, 
one should see the following output, where the version of the XGBoost library is 0.90:

```
Data load time (s):  0.87
[pai4sk - BoostingMachine-CPU] training_time 73.104443 roc_auc_score 0.919776
[XGBoost-CPU] training_time 187.934625 roc_auc_score 0.919320
```

After running the following command on a P9+V100 system ``export OMP_PROC_BIND=spread ; numactl -N0 -m0 python $CONDA_PREFIX/pai4sk/local-examples/example-epsilon/example-epsilon-boosting.py --data_path path/to/data --num_threads 16 --use_gpu``, 
one should see the following output, where the version of the XGBoost library is 0.90:

```
Data load time (s):  0.87
[pai4sk - BoostingMachine-GPU] training_time 34.068972 roc_auc_score 0.919776
[XGBoost-GPU] training_time 50.236198 roc_auc_score 0.920082
```

Times will vary depending on system architecture.

&copy; Copyright IBM Corporation 2018, 2019
