# SUSY Example

In this example we will train Decision Tree and Random Forest models on the SUSY dataset from the LibSVM repository, using both `scikit-learn` and `snap-ml-local`. 

## Download the Data

Download and decompress the data from the LIBSVM repository:
```
mkdir data
cd data
wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/SUSY.bz2
bunzip2 SUSY.bz2
cd ../
```

## PreProcess the data

We first perform some data pre-processing.

The entire pre-processing can be run by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-susy/preprocess-susy.py`

Command to pre-process the data available in specific directory:
`python $CONDA_PREFIX/pai4sk/local-examples/example-susy/preprocess-susy.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.


## Training and Evaluating a Decision Tree Model

Run the example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-decision-tree.py`

Command to run example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-decision-tree.py --data_path path/to/data` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

```
Data load time (s): 0.80
[snap] Training time (s): 22.26
[snap] Accuracy score: 0.7924
[sklearn] Training time (s): 171.95
[sklearn] Accuracy score: 0.7925
```

For the decision tree and random forest models, `snap-ml-local` provides a feature called `histograms`: when enabled, 
the values of each feature are grouped into a user-defined number of bins; during training, the splits are evaluated at the bin values instead
of the individual feature values. This can significantly accelerate the training of the model. To enable `histograms`, the user needs to define two
parameters at model instantiation time: `use_histograms` and `hist_nbins`. An example of how to use these parameters
can be found in `$CONDA_PREFIX/pai4sk/snap-ml-local/local-examples/example-susy/example-susy-decision-tree.py`. The `sklearn` library 
does not offer (as of version 0.22) a decision tree / random forest API with `histograms` functionality. Nevertheless, in this example,
we compare the `snap-ml-local` `histograms`-based decision tree with the `sklearn` decision tree and show the benefits of 
using `histograms` both in terms of training time and generalization performance.

```
[snap.histogram.depth6] Training time (s): 1.79
[snap.histogram.depth6] Accuracy score: 0.7847
[sklearn.depth6] Training time (s): 81.82
[sklearn.depth6] Accuracy score: 0.7848
```

## Training and Evaluating a Random Forest Model

Run the example by executing `python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-random-forest.py`

Command to run example using the pre-processed data in specific directory.
`python $CONDA_PREFIX/pai4sk/local-examples/example-susy/example-susy-random-forest.py --data_path path/to/data --num_threads 10` directory.

Note: By default the location of `--data_path` is current directory. Parent directory of the directory 'data/' is to be given as parameter to --data_path option.

```
Data load time (s):  0.15
[snap] Training time (s):  7.89
[snap] Accuracy score:   0.7962
[sklearn] Training time (s):  33.49
[sklearn] Accuracy score:   0.7954
[snap.histogram.depth6] Training time (s):  2.62
[snap.histogram.depth6] Accuracy score:   0.7853
[sklearn.depth6] Training time (s):  18.03
[sklearn.depth6] Accuracy score:   0.7851
```

Times will vary depending on the system architecture. 

&copy; Copyright IBM Corporation 2018
