# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from __future__ import print_function

import time
import argparse

# pai4sk explicit imports for snap_ml APIs
from pai4sk import MPI
from pai4sk.sml_io import load_from_svmlight_format
from pai4sk import LogisticRegression as snapmlLR
from pai4sk.sml_metrics import log_loss as snapLOGLOSS, accuracy_score as snapACC

# pai4sk explicit imports for sklearn APIs
from pai4sk.linear_model import LogisticRegressionSklearn as sklearnLR
from pai4sk.datasets import load_svmlight_fileSklearn
from pai4sk.metrics import accuracy_scoreSklearn,log_lossSklearn

(comm_size, rank) = MPI.Comm_get_info()

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--data_path",
   type=str,
   default=defaultPath
)
CLI.add_argument(
   "--device_ids",
   nargs="*",
   type=int,
   default=[]
)
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)
args = CLI.parse_args()

if args.use_balanced_class_weights:
  class_weight = "balanced"
else:
  class_weight = None

data = load_from_svmlight_format(args.data_path + "/data/news20.binary")

snap_lr = snapmlLR(device_ids=args.device_ids, use_gpu=args.use_gpu, 
                                   max_iter=50,class_weight=class_weight)

# Training
t0 = time.time()
snap_lr.fit(data)
if (rank == 0):
    print("[pai4sk] MPI process %d : Training time (s):  %.2f" % (rank, time.time()-t0))

proba = snap_lr.predict_proba(data)
pred  = snap_lr.predict(data)

if (rank == 0):
    print("[pai4sk] MPI process %d : Accuracy:       %.4f" % (rank, snapACC(data, pred)))
    print("[pai4sk] MPI process %d : Log loss:       %.4f" % (rank, snapLOGLOSS(data, proba)))

if args.skip_sklearn:
    if (rank == 0):
        print('Execution using scikit-learn is skipped.')
    exit()

X, y = load_svmlight_fileSklearn(args.data_path + "/data/news20.binary")

# Create Logistic Regression model
lr = sklearnLR(dual=True, fit_intercept=False)

t0 = time.time()
lr.fit(X,y)

if (rank == 0):
    print("[sklearn] Training time (s):  %.2f" % (time.time()-t0))

proba = lr.predict_proba(X)
pred  = lr.predict(X)

if (rank == 0):
    print("[sklearn] Accuracy:    %.4f" % (accuracy_scoreSklearn(y, pred)))
    print("[sklearn] Log loss:    %.4f" % (log_lossSklearn(y, proba)))

